#ifndef WIN32
#include <fcntl.h>
#include <errno.h>
#endif
#include <msp/core/systemerror.h>
#include <msp/strings/formatter.h>
#include "handle_private.h"
#include "pipe.h"

using namespace std;

namespace Msp {
namespace IO {

Pipe::Pipe():
	reader(handle[0], 1024)
{
#ifdef WIN32
	string name = format("\\\\.\\pipe\\%u.%p", GetCurrentProcessId(), this);
	*handle[0] = CreateNamedPipe(name.c_str(), PIPE_ACCESS_INBOUND|FILE_FLAG_OVERLAPPED, PIPE_TYPE_BYTE, 1, 1024, 1024, 0, 0);
	if(!handle[0])
		throw system_error("CreateNamedPipe");

	*handle[1] = CreateFile(name.c_str(), GENERIC_WRITE, 0, 0, OPEN_EXISTING, FILE_FLAG_OVERLAPPED, 0);
	if(!handle[1])
	{
		unsigned err = GetLastError();
		CloseHandle(*handle[0]);
		throw system_error(format("CreateFile(%s)", name), err);
	}
#else
	int pipe_fd[2];
	if(pipe(pipe_fd)==-1)
		throw system_error("pipe");

	*handle[0] = pipe_fd[0];
	*handle[1] = pipe_fd[1];
#endif

	set_events(P_INPUT);
}

Pipe::~Pipe()
{
	set_events(P_NONE);

	signal_flush_required.emit();
	sys_close(handle[0]);
	sys_close(handle[1]);
}

void Pipe::set_block(bool b)
{
	mode = (mode&~M_NONBLOCK);
	if(b)
		mode = (mode|M_NONBLOCK);

#ifndef WIN32
	int flags = fcntl(*handle[0], F_GETFD);
	fcntl(*handle[0], F_SETFL, (flags&O_NONBLOCK)|(b?0:O_NONBLOCK));
	flags = fcntl(*handle[1], F_GETFD);
	fcntl(*handle[1], F_SETFL, (flags&O_NONBLOCK)|(b?0:O_NONBLOCK));
#endif
}

unsigned Pipe::do_write(const char *buf, unsigned size)
{
	if(size==0)
		return 0;

	return sys_write(handle[1], buf, size);
}

unsigned Pipe::do_read(char *buf, unsigned size)
{
	if(size==0)
		return 0;

	unsigned ret = reader.read(buf, size);
	if(ret==0)
		set_eof();

	return ret;
}

} // namespace IO
} // namespace Msp
